@@ -26,7 +26,20 @@
 #' @export
 #'
 #'
-LR_database <- function(species, database_name, gene_spot_expression_dataframe, percentage = 10){
+LR_database <- function(species, database_name, gene_spot_expression_dataframe, percentage = NULL){
+  
+  if (is.null(percentage)){
+    percentage <- round((16/ncol(gene_spot_expression_dataframe))*100)
+    if(percentage == 0){
+      percentage <- round((100/ncol(gene_spot_expression_dataframe))*100)
+      if(percentage == 0 ){
+        percentage <- 1
+      }
+    }
+    
+  }
+  
+  
   if ( database_name %in% c("CellChat","cellchat","Cellchat","cellChat")  ){
 
     dat <- possible_L_R_pairs_cellchat(species, gene_spot_expression_dataframe, percentage)
@@ -376,5 +389,79 @@ possible_L_R_pairs_connectome <- function(gene_spot_expression_dataframe, percen
 
 
 
+#' Identify Possible Ligand-Receptor Pairs for Cell-Cell Communication
+#'
+#' Self_identified Database: This function identifies possible ligand-receptor (L-R) pairs for cell-cell communication analysis using data from the self identified database. It checks for the presence of all genes involved in each L-R pair within the provided gene expression matrix, filtering based on a specified expression percentage threshold.
+#'
+#' @param DB_dataframe A dataframe contains the columns of "ligand", "receptor" and "interaction_name
+#' @param gene_spot_expression_dataframe A gene expression data frame with genes as row names and Spot IDs as column names. This data frame is used to verify the presence of all genes involved in the L-R pairs.
+#' @param percentage A numeric value specifying the minimum percentage of spots in which a gene must be expressed to be considered. The default is 10.
+#'
+#' @return A list containing:
+#' \describe{
+#'   \item{possible_L_R_pairs}{A data frame of L-R pairs where all genes are present in the 'gene_spot_expression_dataframe' and meet the expression threshold. The data frame includes the ligand and receptor vectors, and the combined gene vectors.}
+#'   \item{possible_L_R_pairs_details}{A data frame with detailed information about the identified L-R pairs, including their original annotations from the ICELLNET dataset.}
+#' }
+#'
+#' @examples
+#' library(SpaCCI)
+#' #Load the example data
+#' data(test_data)
+#' #Load the example self-identified database
+#' #(for more instructions please check the online tutorial)
+#' data(DB_dataframe)
+#' gene_spot_df <- test_data$gene_spot_df
+#' result <- self_identified_LR( DB_dataframe = DB_dataframe,
+#'                               gene_spot_expression_dataframe = gene_spot_df)
+#'
+#' @export
+#'
+
+
+self_identified_LR <- function(DB_dataframe , gene_spot_expression_dataframe,percentage = NULL){
+  if (is.null(percentage)){
+    percentage <- round((16/ncol(gene_spot_expression_dataframe))*100)
+    if(percentage == 0){
+      percentage <- round((100/ncol(gene_spot_expression_dataframe))*100)
+      if(percentage == 0 ){
+        percentage <- 1
+      }
+    }
+    
+  }
+  
+  # Function to split and trim whitespace
+  split_and_trim <- function(x) {
+    sapply(strsplit(x, ",", fixed = TRUE), function(y) trimws(y))
+  }
+  # Apply the function to ligand and receptor columns
+  DB_dataframe$ligand_vector <- lapply(DB_dataframe$ligand, split_and_trim)
+  DB_dataframe$receptor_vector <- lapply(DB_dataframe$receptor, split_and_trim)
+  
+  # Combine ligand and receptor genes into a single vector for each row
+  DB_dataframe$combined_genes <- mapply(function(l, r) c(l, r),
+                                        DB_dataframe$ligand_vector,
+                                        DB_dataframe$receptor_vector,
+                                        SIMPLIFY = FALSE)
+  # Function to check if all genes of an LR pair are in the data
+  check_genes_presence <- function(gene_vector, gene_data) {
+    all(gene_vector %in% rownames(gene_data))
+  }
+  exp_gene_express <- gene_spot_expression_dataframe[rowSums(gene_spot_expression_dataframe[] )>0,]
+  binary_expression_data <- ifelse(exp_gene_express > 0, 1, 0)
+  expression_percent <- apply(binary_expression_data, 1, function(row) sum(row > 0) / length(row) * 100)
+  expressed_genes <- names(expression_percent)[expression_percent >= percentage ] 
+  final_gene_spot_expression_dataframe <- exp_gene_express[expressed_genes,]
+  
+  presence_vector <- mapply(check_genes_presence, DB_dataframe$combined_genes, MoreArgs = list(gene_data = final_gene_spot_expression_dataframe))
+  
+  # Subset interactions to get only those L-R pairs where all genes are present in gene_data
+  L_R_pairs <- DB_dataframe[presence_vector, c("ligand_vector", "receptor_vector", "combined_genes")]
+  info_inter <- DB_dataframe[presence_vector, setdiff(names(DB_dataframe), c("ligand_vector", "receptor_vector", "combined_genes"))]
+  return(list( possible_L_R_pairs = L_R_pairs, possible_L_R_pairs_details = info_inter))
+  
+}
+
+
 
 
